package tools.descartes.prisma.agent.wildfly.service;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;
import org.jboss.msc.value.InjectedValue;

import edu.kit.ipd.descartes.mm.applicationlevel.functions.DoubleLiteral;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSample;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSampleList;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ProbabilityMassFunction;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.RandomVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ForkAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourceconfiguration.SchedulingPolicy;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.HardwareInfrastructure;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationGroupRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.core.ApplicationModel;
import tools.descartes.prisma.core.RepositoryConnection;
import tools.descartes.prisma.core.templates.ModelTemplate;
import tools.descartes.prisma.core.templates.deployment.DeploymentTemplate;
import tools.descartes.prisma.core.templates.repository.AssemblyContextTemplate;
import tools.descartes.prisma.core.templates.repository.BasicComponentTemplate;
import tools.descartes.prisma.core.templates.repository.CompositeComponentTemplate;
import tools.descartes.prisma.core.templates.repository.InterfaceProvidingRoleTemplate;
import tools.descartes.prisma.core.templates.repository.InterfaceRequiringRoleTemplate;
import tools.descartes.prisma.core.templates.repository.InterfaceTemplate;
import tools.descartes.prisma.core.templates.repository.ProvidingDelegationConnectorTemplate;
import tools.descartes.prisma.core.templates.repository.RepositoryComponentTemplate;
import tools.descartes.prisma.core.templates.repository.RepositoryTemplate;
import tools.descartes.prisma.core.templates.repository.RequiringDelegationConnectorTemplate;
import tools.descartes.prisma.core.templates.repository.SignatureTemplate;
import tools.descartes.prisma.core.templates.resourceconfiguration.ActiveResourceSpecificationTemplate;
import tools.descartes.prisma.core.templates.resourcelandscape.ComputingInfrastructureTemplate;
import tools.descartes.prisma.core.templates.resourcelandscape.DataCenterTemplate;
import tools.descartes.prisma.core.templates.resourcelandscape.DistributedDataCenterTemplate;
import tools.descartes.prisma.core.templates.resourcetype.ProcessingResourceTypeTemplate;
import tools.descartes.prisma.core.templates.resourcetype.ResourceTypeRepositoryTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.ExternalCallActionTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.FineGrainedBehaviorTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.ForkActionTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.InternalActionTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.LoopActionTemplate;
import tools.descartes.prisma.core.templates.system.SystemTemplate;

public class ModelExtractionService implements Service<ModelExtractionService> {
	
	private class CallPaths {
		private Map<String, List<String>> paths = new HashMap<>();
		
		public void addPath(String interf, String assemblyCtx) {
			List<String> a = paths.get(interf);
			if (a == null) {
				a = new LinkedList<>();
				paths.put(interf, a);
			}
			a.add(assemblyCtx);
		}
		
		public int getIndex(String interf, String assemblyCtx) {
			List<String> a = paths.get(interf);
			if (a != null) {
				int idx = 0;
				for (String cur : a) {
					if (cur.equals(assemblyCtx)) {
						return idx;
					}
					idx++;
				}
			}
			return -1;
		}
	}
	
	private static final Logger log = Logger.getLogger(ModelExtractionService.class);
	
	private InjectedValue<MonitoringService> monitoring = new InjectedValue<>();
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "modelextraction");
	
	public static final ModelExtractionService INSTANCE = new ModelExtractionService();

	private RepositoryConnection connection;
	
	private ApplicationModel model;
	
	private Queue<Object> updates = new ConcurrentLinkedQueue<>();
	
	private ModelUpdater updater = new ModelUpdater();
	
	private ComputingInfrastructureTemplate localHost;
	
	private String localIp;
	
	private Map<String, CallPaths> callpaths = new HashMap<>();
	
	private Thread updateThread;
	
	private ModelExtractionService() {
	}
	
	@Override
	public ModelExtractionService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext arg0) throws StartException {
		connection = new RepositoryConnection("10.0.1.148");
		connection.open();
		model = new ApplicationModel(connection, "specjent2010");
		initModel();		
		
		if (updateThread == null) {
			updateThread = new Thread(updater);
			updateThread.start();
		}
	}

	@Override
	public void stop(StopContext arg0) {
		updater.finishUpdates();
		updateThread.interrupt();
		try {
			updateThread.join();
			updateThread = null;
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		try {
			connection.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void initModel() {
		ResourceTypeRepositoryTemplate resourceTypes = new ResourceTypeRepositoryTemplate();
		ProcessingResourceTypeTemplate cpuType = resourceTypes.defineProcessingResource("CPU");
		resourceTypes.defineProcessingResource("DELAY");
		RepositoryTemplate repository = new RepositoryTemplate();
		SystemTemplate system = new SystemTemplate();
		DistributedDataCenterTemplate ddc = new DistributedDataCenterTemplate();
		DataCenterTemplate dataCenter = new DataCenterTemplate("default");
		ddc.add(dataCenter);
		DeploymentTemplate deployment = new DeploymentTemplate();
		
		// Add a computing infrastructure corresponding to the current machine
		try {
			localIp = InetAddress.getLocalHost().getHostName();
			localHost = dataCenter.defineComputingInfrastructure(localIp);
			ActiveResourceSpecificationTemplate res = localHost.defineActiveResource("cpu-resources");
			res.defineProcessingResource("cpu", cpuType, SchedulingPolicy.PROCESSOR_SHARING, Runtime.getRuntime().availableProcessors());
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	
		List<ModelTemplate<?,?>> changes = new LinkedList<>();
		changes.add(resourceTypes);
		changes.add(repository);
		changes.add(ddc);
		changes.add(system);
		changes.add(deployment);
		model.update(changes);		
	}
	
	public void newComponentRecord(ComponentRecord record) {
		updates.add(record);
	}
	
	public void newInterfaceRecord(InterfaceRecord record) {
		updates.add(record);
	}
	
	public void newOperationRecord(OperationRecord record) {
		updates.add(record);
	}
	
	public void newInvocationGroupRecord(InvocationGroupRecord record) {
		updates.add(record);
	}
	
	private class ModelUpdater implements Runnable {
		
		private volatile boolean stop = false;
		
		public void finishUpdates() {
			stop = true;
		}

		@Override
		public void run() {
			RepositoryTemplate repository = new RepositoryTemplate();
			SystemTemplate system = new SystemTemplate();
			DeploymentTemplate deployment = new DeploymentTemplate();

			while(!stop) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
				
				Object current;
				List<ModelTemplate<?,?>> commands = new LinkedList<>();


				boolean changed = false;
				
				while((current = updates.poll()) != null) {
					if (current instanceof InvocationGroupRecord) {
						handleInvocationGroupRecord((InvocationGroupRecord)current, repository, system, deployment);
						changed = true;
					}
				}
				
				if (changed) {
					commands.add(repository);
					commands.add(system);
					commands.add(deployment);
					model.update(commands);	
				}				
			}
		}
		
		private void handleInvocationGroupRecord(InvocationGroupRecord record, RepositoryTemplate repository, SystemTemplate system, DeploymentTemplate deployment) {
			InvocationGroupRecord caller = record.getOperation().getInterface().getComponent().getCaller();			
			if (caller != null) {
				// We have information on the incoming calls as well as the outgoing calls.
				addComponent(repository, system, deployment, caller, record);
			} else {
				// no information about outgoing calls.
				addComponent(repository, system, deployment, record, null);
			}
		}		
	}
	
	private void addComponent(RepositoryTemplate repository, SystemTemplate system, DeploymentTemplate deployment, InvocationGroupRecord incomingGrec, InvocationGroupRecord outgoingGrec) {
		ComponentRecord incomingCrec = incomingGrec.getOperation().getInterface().getComponent();
		String incomingInterfaceName = incomingGrec.getOperation().getInterface().getInterfaceName();
		ComponentRecord outgoingCrec = null;
		String outgoingInterfaceName = null;
		if (outgoingGrec != null) {
			outgoingCrec = outgoingGrec.getOperation().getInterface().getComponent();		
			outgoingInterfaceName = outgoingGrec.getOperation().getInterface().getInterfaceName();
		}
		
		// 1. add the incoming interface to the repository		
		InterfaceTemplate incomingInterface = repository.defineInterface(incomingInterfaceName);
		SignatureTemplate incomingSignature = incomingInterface.defineSignature(getSignatureName(incomingGrec));
		
		// 2. add the outgoing interface to the repository (if applicable)
		InterfaceTemplate outgoingInterface = null;
		SignatureTemplate outgoingSignature = null;
		if (outgoingGrec != null) {
			outgoingInterface = repository.defineInterface(outgoingInterfaceName);
			outgoingSignature = outgoingInterface.defineSignature(getSignatureName(outgoingGrec));
		}	
		
		// 3. Create the implementation component for the caller component
		BasicComponentTemplate implementationComponent = repository.defineBasicComponent(incomingCrec.getComponentClass());
		String providingPortName = getInterfaceRoleName(incomingCrec.getComponentClass(), incomingInterface.getName(), false);
		InterfaceProvidingRoleTemplate callerProvidingRole = implementationComponent.defineInterfaceProvidingRole(providingPortName, incomingInterface);
		InterfaceRequiringRoleTemplate callerRequiringRole = null;
		int callerRoleIdx = 0;
		if (outgoingInterface != null) {
			// Here we check whether there are external calls for the same interface
			// but to different target assembly contexts. If yes, we need to distinguish the two requiring ports
			callerRoleIdx = getRequiringPortIndex(incomingCrec, outgoingGrec.getOperation().getInterface());
			String roleName = getInterfaceRoleName(incomingCrec.getComponentClass(), outgoingInterface.getName(), true);
			if (callerRoleIdx > 0) {
				roleName = roleName + ":" + callerRoleIdx;
			}
			callerRequiringRole = implementationComponent.defineInterfaceRequiringRole(roleName, outgoingInterface);
		}
		FineGrainedBehaviorTemplate behavior = createComponentInternalBehavior(implementationComponent, callerProvidingRole, incomingSignature);
		if (callerRequiringRole != null) {
			addExternalCall(incomingCrec.getComponentClass(), implementationComponent, behavior, callerRequiringRole, outgoingSignature);
		}
		RepositoryComponentTemplate<?> callerComponent = implementationComponent;
		
		// 4. Now add the called component (if applicable)		
		InterfaceProvidingRoleTemplate calledProvidingRole = null;
		RepositoryComponentTemplate<?> calledComponent = null;
		boolean isRemoteCall = false;
		if (callerRequiringRole != null) {
			isRemoteCall = isRemoteCall(incomingCrec, outgoingCrec);
			calledComponent = createCalledComponent(repository, outgoingCrec, isRemoteCall);
			String calledComponentName;
			if (calledComponent instanceof CompositeComponentTemplate) {
				calledComponentName = outgoingCrec.getComponentName();
			} else {
				calledComponentName = outgoingCrec.getComponentClass();
			}
			String calledProvingRoleName = getInterfaceRoleName(calledComponentName, outgoingInterfaceName, false);
			calledProvidingRole = calledComponent.defineInterfaceProvidingRole(calledProvingRoleName, outgoingInterface);
		}
		
		// 5. Now wrap the implementation in a composite component (if necessary)
		CompositeComponentTemplate compositeComponent = createCompositeComponent(repository, incomingCrec, callerComponent);
		if (compositeComponent != null) {

			// Create an assembly context for embedding the component and connect its interface providing role with a role on the composite component
			AssemblyContextTemplate assembly = compositeComponent.defineAssemblyContext(incomingCrec.getComponentName(), implementationComponent);			
			
			String providingRemotePortName = getInterfaceRoleName(incomingCrec.getComponentName(), incomingInterfaceName, false);			
			InterfaceProvidingRoleTemplate remoteProvidingRole = compositeComponent.defineInterfaceProvidingRole(providingRemotePortName, incomingInterface);
			compositeComponent.defineProvidingDelegationConnector(remoteProvidingRole, callerProvidingRole, assembly);
			
			InterfaceRequiringRoleTemplate remoteRequiringRole = null;
			if (callerRequiringRole != null) {
				if (isRemoteCall) {
					// Only add a interface requiring role to the composite component if it is a remote call, i.e., a call to another composite
					// component.					
					String requiringRemotePortName = getInterfaceRoleName(incomingCrec.getComponentName(), outgoingInterfaceName, true);
					if (callerRoleIdx > 0) {
						requiringRemotePortName = requiringRemotePortName + ":" + callerRoleIdx;
					}
					remoteRequiringRole = compositeComponent.defineInterfaceRequiringRole(requiringRemotePortName, outgoingInterface);
					
					compositeComponent.defineRequiringDelegationConnector(remoteRequiringRole, callerRequiringRole, assembly);
				} else {
					// Local call: Also add an assembly context for the called component within the composite component
					// and connect it with an assembly connector.
					AssemblyContextTemplate calledAssembly = compositeComponent.defineAssemblyContext(outgoingCrec.getComponentName(), calledComponent);				
					compositeComponent.defineAssemblyConnector(assembly, callerRequiringRole, calledAssembly, calledProvidingRole);
				}
			}			
			callerProvidingRole = remoteProvidingRole;
			callerRequiringRole = remoteRequiringRole;
			callerComponent = compositeComponent;
		}
		
		// 6. Now create the assembly contexts
		AssemblyContextTemplate callerContext = system.defineAssemblyContext(getAssemblyContextName(callerComponent, incomingCrec), callerComponent);
		if (callerRequiringRole != null) {
			AssemblyContextTemplate calledContext = system.defineAssemblyContext(getAssemblyContextName(calledComponent, outgoingCrec), calledComponent);
			system.defineAssemblyConnector(callerContext, callerRequiringRole, calledContext, calledProvidingRole);
		}
		
		// 7. Expose the provided remote interfaces as system interfaces (if this port is marked as system interface)
		String systemProvidingRoleName = localIp + "/" + incomingInterfaceName;
		// Do not create a new providing/requiring role for the system, only use the existing providing/requiring role.
		// This way a user can control which interfaces are exposed at the system level by creating them manually.
		InterfaceProvidingRoleTemplate systemProvidingRole = new InterfaceProvidingRoleTemplate(ModelTemplate.Status.EXISTS, systemProvidingRoleName, incomingInterface);
		system.defineProvidingDelegationConnector(systemProvidingRole, callerProvidingRole, callerContext);
		if (callerRequiringRole != null) {
			String systemRequiringRoleName = getInterfaceRoleName(incomingCrec.getComponentName(), outgoingInterfaceName, true);
			InterfaceRequiringRoleTemplate systemRequiringRole = new InterfaceRequiringRoleTemplate(ModelTemplate.Status.EXISTS, systemRequiringRoleName, outgoingInterface);
			system.defineRequiringDelegationConnector(systemRequiringRole, callerRequiringRole, callerContext);
		}
		
		// 8. Create the deployment contexts (only for the caller component)
		deployment.defineDeploymentContext(callerContext, localHost);
	}
	
	private FineGrainedBehaviorTemplate createComponentInternalBehavior(BasicComponentTemplate implementationComponent, InterfaceProvidingRoleTemplate implementationProvidingRole, SignatureTemplate signature) {
		FineGrainedBehaviorTemplate behavior = implementationComponent.defineFineGrainedBehavior(implementationProvidingRole, signature);
		if (!isJMSComponent(implementationComponent.getName())) {
			if (behavior.getComponentInternalBehavior().getActions().isEmpty()) {
				InternalActionTemplate internalAction = new InternalActionTemplate(new ProcessingResourceTypeTemplate("CPU"));
				behavior.getComponentInternalBehavior().add(internalAction);
			}
		}
		return behavior;
	}
	
	private String getAssemblyContextName(RepositoryComponentTemplate<?> component, ComponentRecord crec) {
		if (component instanceof CompositeComponentTemplate) {
			// Composite components are named after the component name in the component record.
			// We also use that as the assembly context name.
			return component.getName();
		} else {
			// Basic components are named after the component class in the component record.
			// We use the component name instead for the assembly context since it is used for resolving 
			// remote calls.
			return crec.getComponentName();
		}
	}
	
	private void addExternalCall(String componentName, 
			BasicComponentTemplate implementationComponent, 
			FineGrainedBehaviorTemplate behavior, 
			InterfaceRequiringRoleTemplate requiringRole, 
			SignatureTemplate outgoingSignature) {		
		ExternalCallActionTemplate call = new ExternalCallActionTemplate(requiringRole, outgoingSignature);
		if (isJMSComponent(componentName)) {
			if (behavior.getComponentInternalBehavior().getActions().isEmpty()) {
				ForkActionTemplate fork = new ForkActionTemplate(false, 1);
				fork.getForkedBehavior(0).add(call);
				behavior.getComponentInternalBehavior().add(fork);
			}
		} else {
			LoopActionTemplate loop = new LoopActionTemplate();
			loop.getLoopBehavior().add(call);
			behavior.getComponentInternalBehavior().add(loop);
		}
	}
	
	private CompositeComponentTemplate createCompositeComponent(RepositoryTemplate repository, ComponentRecord crec, RepositoryComponentTemplate<?> implementationComponent) {
		String compositeName = getCompositeComponentName(crec);
		if (compositeName != null) {
			return repository.defineCompositeComponent(compositeName);
		}
		return null;
	}
	
	private RepositoryComponentTemplate<?> createCalledComponent(RepositoryTemplate repository, ComponentRecord callee, boolean remoteCall) {
		// check whether this call goes to a component in the same composite or not
		if (remoteCall) {
			String calleeCompositeName = getCompositeComponentName(callee);
			if (calleeCompositeName != null) {
				return repository.defineCompositeComponent(calleeCompositeName);
			}
		}
		if (callee.getComponentClass() != null) {
			return repository.defineBasicComponent(callee.getComponentClass());
		}
		throw new IllegalStateException("Could not create called component for " + callee.getComponentName());
	}
	
	private boolean isRemoteCall(ComponentRecord caller, ComponentRecord callee) {
		String callerCompositeName = getCompositeComponentName(caller);
		String calleeCompositeName = getCompositeComponentName(callee);
		// A call is considered local if the two components are in the same composite		
		return (calleeCompositeName == null) || (callerCompositeName == null) || !calleeCompositeName.equals(callerCompositeName);
	}

	private boolean isJMSComponent(String componentName) {
		return "javax.jms.Queue".equals(componentName) || "javax.jms.Topic".equals(componentName);
	}
	
	private String getCompositeComponentName(ComponentRecord crec) {
		String componentPath = crec.getComponentName();
		if (componentPath.startsWith("/")) {
			componentPath = componentPath.substring(1);
		}
		if (componentPath.endsWith("/")) {
			componentPath = componentPath.substring(0, componentPath.length() - 1);
		}
		int idx = componentPath.lastIndexOf("/");
		if (idx < 0) {
			return null;
		} else {
			return componentPath.substring(0, idx);
		}
	}
	
	private int getRequiringPortIndex(ComponentRecord incomingCrec, InterfaceRecord outgoingIrec) {
		CallPaths paths = callpaths.get(incomingCrec.getComponentName());
		if (paths == null) {
			paths = new CallPaths();
			callpaths.put(incomingCrec.getComponentName(), paths);
		}
		paths.addPath(outgoingIrec.getInterfaceName(), outgoingIrec.getComponent().getComponentName());
		int idx = paths.getIndex(outgoingIrec.getInterfaceName(), outgoingIrec.getComponent().getComponentName());	
		if (idx < 0) {
			throw new IllegalStateException();
		}
		return idx;
	}
	
	public InjectedValue<MonitoringService> getMonitoringService() {
		return monitoring;
	}
	
	private String getSignatureName(InvocationGroupRecord record) {
		String name = record.getOperation().getInterface().getInterfaceName() + "#" + record.getOperation().getOperationName();
		if (!record.getName().isEmpty()) {
			name += "_" + record.getName();
		}
		return name;
	}
	

	private String getInterfaceRoleName(String component, String interf, boolean req) {
		return component + "#" + (req ? "Req:" : "Prov:") + interf;
	}
}

