package tools.descartes.prisma.core;

import static org.fest.assertions.api.Assertions.assertThat;

import org.eclipse.emf.ecore.resource.Resource;
import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.usageprofile.UsageProfile;

public class ApplicationScopeTest extends ContainerScopeTest<ApplicationScope> {

	@Before
	public void setUp() {
		name = "test";
		repository = new TestModelRepository();
		repository.getSystemScope().create();
		Scope scope = repository.getScope(ApplicationScope.APPLICATION_SCOPE_TYPE + "/" + name);
		assertThat(scope).isInstanceOf(ApplicationScope.class);
		scope.create();
		this.scope = (ApplicationScope) scope;
	}

	@Test
	public void testComponentRepository() {
		Resource repositoryRes = repository.getResource(scope.getFolder() + "/" + name + ".repository");
		assertThat(repositoryRes).isNotNull();
		assertThat(repositoryRes.getContents()).hasSize(1);
		Repository compRepository = (Repository) repositoryRes.getContents().get(0);
		assertThat(compRepository).isNotNull();
		assertThat(compRepository.getName()).isEqualTo(name);
		assertThat(compRepository.getComponents()).isEmpty();
		assertThat(compRepository.getDatatypes()).isEmpty();
		assertThat(compRepository.getInterfaces()).isEmpty();
	}

	@Test
	public void testDeployment() {
		Resource deploymentRes = repository.getResource(scope.getFolder() + "/" + name + ".deployment");
		assertThat(deploymentRes).isNotNull();
		assertThat(deploymentRes.getContents()).hasSize(1);
		Deployment deployment = (Deployment) deploymentRes.getContents().get(0);
		assertThat(deployment).isNotNull();
		assertThat(deployment.getName()).isEqualTo(name);
		assertThat(deployment.getTargetResourceLandscape())
				.isEqualTo(repository.getSystemScope().getResourceLandscape());
		assertThat(deployment.getSystem()).isEqualTo(repository.getSystemScope().getSystem());
		assertThat(deployment.getDeploymentContexts()).isEmpty();
	}

	@Test
	public void testUsageProfile() {
		Resource usageProfileRes = repository.getResource(scope.getFolder() + "/" + name + ".usageprofile");
		assertThat(usageProfileRes).isNotNull();
		assertThat(usageProfileRes.getContents()).hasSize(1);
		UsageProfile usageProfile = (UsageProfile) usageProfileRes.getContents().get(0);
		assertThat(usageProfile).isNotNull();
		assertThat(usageProfile.getName()).isEqualTo(name);
		assertThat(usageProfile.getSystem()).isEqualTo(repository.getSystemScope().getSystem());
		assertThat(usageProfile.getUsageScenarios()).isEmpty();
	}

	@Test
	public void testApplicationResources() {
		Resource resourceTypeRes = repository.getResource(scope.getFolder() + "/" + name + ".resourcetypes");
		assertThat(resourceTypeRes).isNotNull();
		assertThat(resourceTypeRes.getContents()).hasSize(1);
		ResourceTypeRepository resourceTypes = (ResourceTypeRepository) resourceTypeRes.getContents().get(0);
		assertThat(resourceTypes).isNotNull();
		assertThat(resourceTypes.getResourceTypes()).isEmpty();
	}

}
