package tools.descartes.prisma.core;

import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypeFactory;
import edu.kit.ipd.descartes.mm.usageprofile.UsageProfile;
import edu.kit.ipd.descartes.mm.usageprofile.UsageProfileFactory;

public class ApplicationScope extends ContainerScope {

	public static final String APPLICATION_SCOPE_TYPE = "application";

	private static final String REPOSITORY_SUFFIX = ".repository";
	private static final String RESOURCE_TYPES_SUFFIX = ".resourcetypes";
	private static final String USAGE_PROFILE_SUFFIX = ".usageprofile";
	private static final String DEPLOYMENT_SUFFIX = ".deployment";

	private Repository componentRepository;
	private ResourceTypeRepository applicationResourceTypes;
	private UsageProfile usageProfile;
	private Deployment deployment;

	public ApplicationScope(String name, SystemGlobalScope systemScope, ModelRepository repository) {
		super(name, APPLICATION_SCOPE_TYPE, systemScope, repository);
	}

	public Repository getComponentRepository() {
		if (componentRepository == null) {
			componentRepository = (Repository) getModelRepository().getResource(getComponentRepositoryPath()).getContents()
					.get(0);
		}
		return componentRepository;
	}

	private String getComponentRepositoryPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + REPOSITORY_SUFFIX;
	}

	public ResourceTypeRepository getApplicationResourceTypes() {
		if (applicationResourceTypes == null) {
			applicationResourceTypes = (ResourceTypeRepository) getModelRepository()
					.getResource(getApplicationResourceTypesPath()).getContents().get(0);
		}
		return applicationResourceTypes;
	}

	private String getApplicationResourceTypesPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + RESOURCE_TYPES_SUFFIX;
	}

	public UsageProfile getUsageProfile() {
		if (usageProfile == null) {
			usageProfile = (UsageProfile) getModelRepository().getResource(getUsageProfilePath()).getContents().get(0);
		}
		return usageProfile;
	}

	private String getUsageProfilePath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + USAGE_PROFILE_SUFFIX;
	}

	public Deployment getDeployment() {
		if (deployment == null) {
			deployment = (Deployment) getModelRepository().getResource(getDeploymentPath()).getContents().get(0);
		}
		return deployment;
	}

	private String getDeploymentPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + DEPLOYMENT_SUFFIX;
	}

	@Override
	protected void createResources(Transaction transaction) {
		super.createResources(transaction);
		
		DistributedDataCenter ddc = getSystemScope().getResourceLandscape();
		System containingSystem = getSystemScope().getSystem();
		
		Resource resourceTypesResource = getModelRepository().getOrCreateResource(transaction,
				getApplicationResourceTypesPath());
		if (resourceTypesResource.getContents().isEmpty()) {
			ResourceTypeRepository resourceTypes = ResourcetypeFactory.eINSTANCE.createResourceTypeRepository();
			resourceTypesResource.getContents().add(resourceTypes);
		}

		Resource repositoryResource = getModelRepository().getOrCreateResource(transaction,
				getComponentRepositoryPath());
		if (repositoryResource.getContents().isEmpty()) {
			Repository repository = RepositoryFactory.eINSTANCE.createRepository();
			repository.setName(getName());
			repositoryResource.getContents().add(repository);
		}

		Resource deploymentResource = getModelRepository().getOrCreateResource(transaction, getDeploymentPath());
		if (deploymentResource.getContents().isEmpty()) {
			Deployment deployment = DeploymentFactory.eINSTANCE.createDeployment();
			deployment.setName(getName());
			deployment.setSystem(containingSystem);
			deployment.setTargetResourceLandscape(ddc);
			deploymentResource.getContents().add(deployment);
		}

		Resource usageProfileResource = getModelRepository().getOrCreateResource(transaction, getUsageProfilePath());
		if (usageProfileResource.getContents().isEmpty()) {
			UsageProfile usageProfile = UsageProfileFactory.eINSTANCE.createUsageProfile();
			usageProfile.setName(getName());
			usageProfile.setSystem(containingSystem);
			usageProfileResource.getContents().add(usageProfile);
		}
	}

}
