package tools.descartes.prisma.core;

import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapeFactory;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypeFactory;

public class SystemGlobalScope extends Scope {
	
	private static final String SYSTEM_GLOBAL_SCOPE_NAME = "global";
	private static final String REPOSITORY_SUFFIX = ".repository";
	private static final String RESOURCE_TYPES_SUFFIX = ".resourcetypes";
	private static final String SYSTEM_SUFFIX = ".system";
	private static final String RESOURCELANDSCAPE_SUFFIX = ".resourcelandscape";
	
	private System system;
	private Repository componentRepository;
	private ResourceTypeRepository resourceTypes;
	private DistributedDataCenter infrastructureResourceLandscape;

	public SystemGlobalScope(ModelRepository repository) {
		super(SYSTEM_GLOBAL_SCOPE_NAME, repository);
	}
	
	@Override
	protected String getBasePath() {
		return "";
	}

	public Repository getComponentRepository() {
		if (componentRepository == null) {
			componentRepository = (Repository) getModelRepository().getResource(getComponentRepositoryPath()).getContents()
					.get(0);
		}
		return componentRepository;
	}

	private String getComponentRepositoryPath() {
		return getName() + "/" + getName() + REPOSITORY_SUFFIX;
	}
	
	public System getSystem() {
		if (system == null) {
			system = (System) getModelRepository().getResource(getSystemPath()).getContents().get(0);
		}
		return system;
	}

	private String getSystemPath() {
		return getName() + "/" + getName() + SYSTEM_SUFFIX;
	}
	
	public ResourceTypeRepository getResourceTypes() {
		if (resourceTypes == null) {
			resourceTypes = (ResourceTypeRepository) getModelRepository().getResource(getResourceTypesPath())
					.getContents().get(0);
		}
		return resourceTypes;
	}

	private String getResourceTypesPath() {
		return getName() + "/" + getName() + RESOURCE_TYPES_SUFFIX;
	}
	
	public DistributedDataCenter getResourceLandscape() {
		if (infrastructureResourceLandscape == null) {
			infrastructureResourceLandscape = (DistributedDataCenter) getModelRepository()
					.getResource(getResourceLandscapePath()).getContents().get(0);
		}
		return infrastructureResourceLandscape;
	}

	private String getResourceLandscapePath() {
		return getName() + "/" + getName() + RESOURCELANDSCAPE_SUFFIX;
	}

	@Override
	protected void createResources(Transaction transaction) {
		Resource globalResourceTypesResource = getModelRepository().getOrCreateResource(transaction,
				getResourceTypesPath());
		if (globalResourceTypesResource.getContents().isEmpty()) {
			ResourceTypeRepository globalResourceTypes = ResourcetypeFactory.eINSTANCE.createResourceTypeRepository();
			globalResourceTypesResource.getContents().add(globalResourceTypes);
		}
		
		Resource systemResource = getModelRepository().getOrCreateResource(transaction, getSystemPath());
		if (systemResource.getContents().isEmpty()) {
			System system = SystemFactory.eINSTANCE.createSystem();
			system.setName(getName());
			systemResource.getContents().add(system);
		}
		
		Resource repositoryResource = getModelRepository().getOrCreateResource(transaction,
				getComponentRepositoryPath());
		if (repositoryResource.getContents().isEmpty()) {
			Repository globalRepository = RepositoryFactory.eINSTANCE.createRepository();
			globalRepository.setName(getName());
			repositoryResource.getContents().add(globalRepository);
		}

		Resource ddcResource = getModelRepository().getOrCreateResource(transaction, getResourceLandscapePath());
		if (ddcResource.getContents().isEmpty()) {
			DistributedDataCenter ddc = ResourcelandscapeFactory.eINSTANCE.createDistributedDataCenter();
			ddc.setName(getName());
			ddcResource.getContents().add(ddc);
		}
	}

}
