package tools.descartes.prisma.core.impl;

import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.cdo.util.CommitException;
import org.eclipse.emf.ecore.EObject;

import tools.descartes.prisma.core.Transaction;

public class CDOTransactionWrapper implements Transaction {

	private final CDOModelRepository repository;
	private final CDOTransaction transaction;

	protected CDOTransactionWrapper(CDOModelRepository repository, CDOTransaction transaction) {
		this.repository = repository;
		this.transaction = transaction;
	}

	@Override
	public void close() {
		transaction.close();
	}

	@Override
	public void commit() {
		try {
			transaction.commit();
		} catch(CommitException ex) {
			throw new tools.descartes.prisma.core.exceptions.CommitException(ex);
		}
	}

	@Override
	public void rollback() {
		transaction.rollback();
	}

	@Override
	public <T extends EObject> T attach(T object) {
		return transaction.getObject(object);
	}

	@Override
	public <T extends EObject> T detach(T object) {
		return repository.getReadOnlyView().getObject(object);
	}

	protected CDOTransaction getWrappedTransaction() {
		return transaction;
	}
}
