package tools.descartes.prisma.core.templates.deployment;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.deployment.DeploymentPackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.Container;
import tools.descartes.prisma.core.templates.core.NamedElementTemplate;
import tools.descartes.prisma.core.templates.repository.AssemblyContextTemplate;
import tools.descartes.prisma.core.templates.resourcelandscape.ComputingInfrastructureTemplate;

public class DeploymentContextTemplate extends NamedElementTemplate<Deployment, DeploymentContext> {
	
	private final AssemblyContextTemplate assemblyContextTemplate;
	private final ComputingInfrastructureTemplate containerTemplate;

	public DeploymentContextTemplate(AssemblyContextTemplate assemblyContextTemplate, ComputingInfrastructureTemplate containerTemplate) {
		super(DeploymentPackage.Literals.DEPLOYMENT__DEPLOYMENT_CONTEXTS, DeploymentContext.class, assemblyContextTemplate.getName() + "@" + containerTemplate.getName());
		this.assemblyContextTemplate = assemblyContextTemplate;
		this.containerTemplate = containerTemplate;
	}

	@Override
	public boolean matches(DeploymentContext object) {
		if (object != null) {
			return super.matches(object)
					&& assemblyContextTemplate.matches(object.getAssemblyContext());
		}
		return false;
	}

	@Override
	protected DeploymentContext create(Deployment parent) {
		AssemblyContext assembly = assemblyContextTemplate.get(getContext().getSystem());
		Container container = containerTemplate.get(getContext().getCurrentDataCenter());
		DeploymentContext ctx = DeploymentFactory.eINSTANCE.createDeploymentContext();
		ctx.setName(getName());
		ctx.setAssemblyContext(assembly);
		ctx.setResourceContainer(container);
		return ctx;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((assemblyContextTemplate == null) ? 0 : assemblyContextTemplate.hashCode());
		result = prime * result + ((containerTemplate == null) ? 0 : containerTemplate.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		DeploymentContextTemplate other = (DeploymentContextTemplate) obj;
		if (assemblyContextTemplate == null) {
			if (other.assemblyContextTemplate != null)
				return false;
		} else if (!assemblyContextTemplate.equals(other.assemblyContextTemplate))
			return false;
		if (containerTemplate == null) {
			if (other.containerTemplate != null)
				return false;
		} else if (!containerTemplate.equals(other.containerTemplate))
			return false;
		return true;
	}
}
