package tools.descartes.prisma.core.templates.repository;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ComposedStructure;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import tools.descartes.prisma.core.templates.core.NamedElementTemplate;

public class AssemblyContextTemplate extends NamedElementTemplate<ComposedStructure, AssemblyContext> {
	
	private final RepositoryComponentTemplate<?> component;
	
	public AssemblyContextTemplate(String assemblyContextName, RepositoryComponentTemplate<?> component) {
		super(RepositoryPackage.Literals.COMPOSED_STRUCTURE__ASSEMBLY_CONTEXTS, AssemblyContext.class, assemblyContextName);
		this.component = component;
	}	

	@Override
	public boolean matches(AssemblyContext object) {
		if (object != null) {
			// If the user changed something and left the model in an inconsistent state,
			// the encapsulated component may be null --> ignore such incorrect entries!
			return super.matches(object)
					&& (object.getEncapsulatedComponent() != null);
		}
		return false;
	}

	@Override
	protected AssemblyContext create(ComposedStructure parent) {
		RepositoryComponent encapsulated = component.get(getContext().getRepository());
		AssemblyContext assembly = RepositoryFactory.eINSTANCE.createAssemblyContext();
		assembly.setId("AssemblyContext_" + getId(getName()));
		assembly.setName(getName());
		RepositoryComponent current = assembly.getEncapsulatedComponent();
		if (current != encapsulated) {
			if (current != null) {
				// TODO logging
				//log.warn("Encapsulated component of assembly context " + assemblyContextName + " will be overwritten (current=" + current.getName() + ", new=" + encapsulated.getName() + ").");
			}
			assembly.setEncapsulatedComponent(encapsulated);
		}
		return assembly;
	}

}
