package tools.descartes.prisma.core.templates.repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import tools.descartes.prisma.core.templates.ModelTemplate;
import tools.descartes.prisma.core.templates.servicebehavior.FineGrainedBehaviorTemplate;

public class BasicComponentTemplate extends RepositoryComponentTemplate<BasicComponent> {
	
	private static class BehaviorKey {
		public final InterfaceProvidingRoleTemplate providingRole;
		public final SignatureTemplate signature;
		public BehaviorKey(InterfaceProvidingRoleTemplate providingRole, SignatureTemplate signature) {
			super();
			this.providingRole = providingRole;
			this.signature = signature;
		}
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((providingRole == null) ? 0 : providingRole.hashCode());
			result = prime * result + ((signature == null) ? 0 : signature.hashCode());
			return result;
		}
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			BehaviorKey other = (BehaviorKey) obj;
			if (providingRole == null) {
				if (other.providingRole != null)
					return false;
			} else if (!providingRole.equals(other.providingRole))
				return false;
			if (signature == null) {
				if (other.signature != null)
					return false;
			} else if (!signature.equals(other.signature))
				return false;
			return true;
		}		
	}
	
	private final Map<BehaviorKey, FineGrainedBehaviorTemplate> fineGrainedBehaviors = new HashMap<>();

	public BasicComponentTemplate(String componentName) {
		super(componentName, BasicComponent.class);
	}
	
	@Override
	public List<ModelTemplate<? super BasicComponent, ?>> getChildCommands() {
		List<ModelTemplate<? super BasicComponent, ?>> list = super.getChildCommands();
		list.addAll(fineGrainedBehaviors.values());
		return list;
	}
	
	@Override
	protected BasicComponent createRepositoryComponent() {
		return RepositoryFactory.eINSTANCE.createBasicComponent();
	}
	
	public FineGrainedBehaviorTemplate defineFineGrainedBehavior(InterfaceProvidingRoleTemplate providingRole, SignatureTemplate signature) {
		BehaviorKey key = new BehaviorKey(providingRole, signature);
		FineGrainedBehaviorTemplate template = fineGrainedBehaviors.get(key);
		if (template == null) {
			template = new FineGrainedBehaviorTemplate(providingRole, signature);
			fineGrainedBehaviors.put(key, template);
		}
		return template;
	}

}
