package tools.descartes.prisma.core.templates.repository;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import tools.descartes.prisma.core.templates.ModelTemplate;
import tools.descartes.prisma.core.templates.core.NamedElementTemplate;

public abstract class RepositoryComponentTemplate<E extends RepositoryComponent> extends NamedElementTemplate<Repository, E> {
	
	private final Map<String, InterfaceProvidingRoleTemplate> interfaceProvidingRoles = new HashMap<>();
	private final Map<String, InterfaceRequiringRoleTemplate> interfaceRequiringRoles = new HashMap<>();

	protected RepositoryComponentTemplate(String componentName, Class<E> componentType) {
		super(RepositoryPackage.Literals.REPOSITORY__COMPONENTS, componentType, componentName);
	}
	
	@Override
	public E create(Repository parent) {
		E component = createRepositoryComponent();
		component.setId("Component_" + getId(getName()));
		component.setName(getName());
		return component;
	}
	
	@Override
	public List<ModelTemplate<? super E, ?>> getChildCommands() {
		List<ModelTemplate<? super E, ?>> list = new LinkedList<>();
		list.addAll(interfaceProvidingRoles.values());
		list.addAll(interfaceRequiringRoles.values());
		return list;
	}
	
	protected abstract E createRepositoryComponent();
	
	public InterfaceProvidingRoleTemplate defineInterfaceProvidingRole(String portName, InterfaceTemplate interf) {
		InterfaceProvidingRoleTemplate template = interfaceProvidingRoles.get(portName);
		if (template == null) {
			template = new InterfaceProvidingRoleTemplate(portName, interf);
			interfaceProvidingRoles.put(portName, template);
		}
		return template;
	}
	
	public InterfaceRequiringRoleTemplate defineInterfaceRequiringRole(String portName, InterfaceTemplate interf) {
		InterfaceRequiringRoleTemplate template = interfaceRequiringRoles.get(portName);
		if (template == null) {
			template = new InterfaceRequiringRoleTemplate(portName, interf);
			interfaceRequiringRoles.put(portName, template);
		}
		return template;
	}
}
