package tools.descartes.prisma.core.templates.repository;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import tools.descartes.prisma.core.templates.ModelTemplate;

public class RepositoryTemplate extends ModelTemplate<Repository, Repository> {
	
	public RepositoryTemplate() {
		super(null, Repository.class);
	}

	private final Map<String, InterfaceTemplate> interfaces = new HashMap<>();
	private final Map<String, RepositoryComponentTemplate<?>> components = new HashMap<>();
	
	@Override
	public Repository get(Repository parent) {
		Repository repo = find(getContext().getRepositoryResource().getContents());
		if (repo == null) {
			repo = create(null);
			getContext().getRepositoryResource().getContents().add(repo);
		}
		return repo;
	}
	
	public InterfaceTemplate defineInterface(String name) {
		InterfaceTemplate template = interfaces.get(name);
		if (template == null) {
			template = new InterfaceTemplate(name);
			interfaces.put(name, template);
		}
		return template;
	}
	
	public BasicComponentTemplate defineBasicComponent(String name) {
		BasicComponentTemplate template = (BasicComponentTemplate)components.get(name);
		if (template == null) {
			template = new BasicComponentTemplate(name);
			components.put(name, template);
		}
		return template;
	}
	
	public CompositeComponentTemplate defineCompositeComponent(String name) {
		CompositeComponentTemplate template = (CompositeComponentTemplate)components.get(name);
		if (template == null) {
			template = new CompositeComponentTemplate(name);
			components.put(name, template);
		}
		return template;
	}
	
	@Override
	public List<ModelTemplate<Repository, ?>> getChildCommands() {
		List<ModelTemplate<Repository,?>> children = new LinkedList<>();
		children.addAll(interfaces.values());
		children.addAll(components.values());
		return children;
	}


	@Override
	public boolean matches(Repository object) {
		return object.getName().equals(getContext().getApplicationName());
	}


	@Override
	protected Repository create(Repository parent) {
		Repository repository = RepositoryFactory.eINSTANCE.createRepository();
		repository.setName(getContext().getApplicationName());
		return repository;
	}

}
