package tools.descartes.prisma.core.templates.servicebehavior;

import java.util.ArrayList;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.AbstractAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BranchAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BranchProbabilities;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import tools.descartes.prisma.core.templates.ModelTemplate;

public abstract class AbstractActionTemplate<E extends AbstractAction> extends ModelTemplate<ComponentInternalBehavior, E> {
	
	private AbstractActionTemplate<?> predecessor;	
	
	protected AbstractActionTemplate(Class<E> actionType) {
		super(ServicebehaviorPackage.Literals.COMPONENT_INTERNAL_BEHAVIOR__ACTIONS, actionType);
	}
	
	@Override
	public E get(ComponentInternalBehavior parent) {
		// 1. check whether it is empty --> simply add the action and return
		if (parent.getActions().isEmpty()) {
			if (getStatus() == Status.CREATE) {
				E newAction = create(parent);
				parent.getActions().add(newAction);
				return newAction;
			} else {
				return null;
			}
		}
		
		// 2. Determine the start point from which we begin with the search
		// 	  search from the beginning
		ComponentInternalBehavior currentBehavior = parent;
		int positionInBehavior = 0;
		if (predecessor != null) {
			AbstractAction predecessorAction = predecessor.get(parent);
			if (predecessorAction != null) {
				currentBehavior = predecessorAction.getParentBehavior();
				int idx = currentBehavior.getActions().indexOf(predecessorAction);
				if ((idx + 1) < currentBehavior.getActions().size()) {
					positionInBehavior = idx + 1;
				} else {
					// the new action is appended behind the predecessor
					if (getStatus() == Status.CREATE) {
						E newAction = create(currentBehavior);
						currentBehavior.getActions().add(newAction);
						return newAction;
					} else {
						return null;
					}
				}
			}			
		}
		
		// 4. insert into existing behavior
		AbstractAction insertionPlace = currentBehavior.getActions().get(positionInBehavior);
		if (insertionPlace instanceof BranchAction) {
			BranchAction branch = (BranchAction)insertionPlace;
			// The branch actions are automatically inserted
			// if there alternative call paths were monitored.
			// In this case we need to check whether there is
			// a already a branch with this action as first entry
			for (ComponentInternalBehavior branchBehavior : branch.getBranches()) {
				if (!branchBehavior.getActions().isEmpty()) {
					E matchedAction = matchesWithCast(branchBehavior.getActions().get(0));
					if (matchedAction != null) {
						return matchedAction;
					}
				}
			}
			if (getStatus() == Status.CREATE) {
				E newAction = create(parent);
				ComponentInternalBehavior newBranchBehavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
				newBranchBehavior.getActions().add(newAction);
				branch.getBranches().add(newBranchBehavior);
				return newAction;
			}
			return null;
		} else {
			// Check the current action for a match
			// If the two do not match we need to introduce a 
			// branch at the current position.
			E existingAction = this.matchesWithCast(insertionPlace);
			if (existingAction != null) {
				return existingAction;
			} else if (getStatus() == Status.CREATE) {
				// The two actions are conflicting --> introduce a new branch
				E newAction = create(parent);
				BranchAction currentBranchAction = insertNewBranchingAction(currentBehavior, positionInBehavior);
				ComponentInternalBehavior newBranchBehavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
				newBranchBehavior.getActions().add(newAction);
				currentBranchAction.getBranches().add(newBranchBehavior);
				return newAction;
			} else {
				return null;
			}
		}
	}
	
	protected void setPredecessorAction(AbstractActionTemplate<?> predecessor) {
		this.predecessor = predecessor;
	}
	
	private BranchAction insertNewBranchingAction(ComponentInternalBehavior parent, int positionInBehavior) {
		BranchAction newBranchAction = ServicebehaviorFactory.eINSTANCE.createBranchAction();
		BranchProbabilities probabilities = ServicebehaviorFactory.eINSTANCE.createBranchProbabilities();
		probabilities.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
		newBranchAction.setBranchProbabilities(probabilities);
		
		
		int numActions = parent.getActions().size() - positionInBehavior;
		List<AbstractAction> movedActions = new ArrayList<>(numActions);
		for (int i = 0; i < numActions; i++) {
			movedActions.add(parent.getActions().get(positionInBehavior + i));
		}
		parent.getActions().removeAll(movedActions);
		ComponentInternalBehavior existingBranchBehavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
		existingBranchBehavior.getActions().addAll(movedActions);		
		newBranchAction.getBranches().add(existingBranchBehavior);
		parent.getActions().add(newBranchAction);
		
		return newBranchAction;
	}
}
