package tools.descartes.prisma.core.templates.servicebehavior;

import java.util.Collections;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import tools.descartes.prisma.core.templates.ModelTemplate;
import tools.descartes.prisma.core.templates.repository.InterfaceProvidingRoleTemplate;
import tools.descartes.prisma.core.templates.repository.SignatureTemplate;

public class FineGrainedBehaviorTemplate extends ModelTemplate<BasicComponent, FineGrainedBehavior> {

	private final ComponentInternalBehaviorTemplate<FineGrainedBehavior> internalBehaviorCmd = new ComponentInternalBehaviorTemplate<>(ServicebehaviorPackage.Literals.FINE_GRAINED_BEHAVIOR__BEHAVIOR);
	private final InterfaceProvidingRoleTemplate addProvidingRoleCmd;
	private final SignatureTemplate addSignatureCmd;

	public FineGrainedBehaviorTemplate(InterfaceProvidingRoleTemplate addProvidingRole, SignatureTemplate addSignature) {
		super(RepositoryPackage.Literals.BASIC_COMPONENT__FINE_GRAINED_BEHAVIOR, FineGrainedBehavior.class);
		this.addProvidingRoleCmd = addProvidingRole;
		this.addSignatureCmd = addSignature;
	}

	@Override
	public boolean matches(FineGrainedBehavior object) {
		return addProvidingRoleCmd.matches(object.getInterfaceProvidingRole())
				&& addSignatureCmd.matches(object.getDescribedSignature());
	}

	@Override
	protected FineGrainedBehavior create(BasicComponent parent) {
		InterfaceProvidingRole providingRole = addProvidingRoleCmd.get(parent);
		Signature signature = addSignatureCmd.get(providingRole.getInterface());
		
		FineGrainedBehavior behavior = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
		behavior.setInterfaceProvidingRole(providingRole);
		behavior.setDescribedSignature(signature);
		behavior.setBehavior(ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior());
		return behavior;
	}
	
	@Override
	public List<? extends ModelTemplate<? super FineGrainedBehavior, ?>> getChildCommands() {
		return Collections.singletonList(internalBehaviorCmd);
	}
	
	public ComponentInternalBehaviorTemplate<FineGrainedBehavior> getComponentInternalBehavior() {
		return internalBehaviorCmd;
	}
}
