package tools.descartes.prisma.core.templates.servicebehavior;

import java.util.ArrayList;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ForkAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import tools.descartes.prisma.core.templates.ModelTemplate;

public class ForkActionTemplate extends AbstractActionTemplate<ForkAction> {
	
	private final boolean withSync;
	private final List<ComponentInternalBehaviorTemplate<ForkAction>> forkBehaviorCmd;

	public ForkActionTemplate(boolean withSync, int threads) {
		super(ForkAction.class);
		this.withSync = withSync;
		forkBehaviorCmd = new ArrayList<>(threads);
		for (int i = 0; i < threads; i++) {
			forkBehaviorCmd.add(new ComponentInternalBehaviorTemplate<ForkAction>(ServicebehaviorPackage.Literals.FORK_ACTION__FORKED_BEHAVIORS));
		}
	}
	
	@Override
	public List<? extends ModelTemplate<? super ForkAction, ?>> getChildCommands() {
		return forkBehaviorCmd;
	}

	@Override
	public boolean matches(ForkAction object) {
		if (object.isWithSynchronizationBarrier() != withSync) {
			return false;
		}
		if (object.getForkedBehaviors().size() != forkBehaviorCmd.size()) {
			return false;
		}
		for (int i = 0; i < forkBehaviorCmd.size(); i++) {
			if (!forkBehaviorCmd.get(i).matches(object.getForkedBehaviors().get(i))) {
				return false;
			}
		}
		return true;
	}

	@Override
	protected ForkAction create(ComponentInternalBehavior parent) {
		ForkAction fork = ServicebehaviorFactory.eINSTANCE.createForkAction();
		fork.setWithSynchronizationBarrier(withSync);
		return fork;
	}
	
	public ComponentInternalBehaviorTemplate<ForkAction> getForkedBehavior(int idx) {
		return forkBehaviorCmd.get(idx);
	}
}
