package tools.descartes.prisma.core.templates.servicebehavior;

import java.util.Arrays;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.resourcetype.ProcessingResourceType;
import tools.descartes.prisma.core.templates.resourcetype.ProcessingResourceTypeTemplate;

public class InternalActionTemplate extends AbstractActionTemplate<InternalAction> {
	
	private final List<ProcessingResourceTypeTemplate> resources;

	public InternalActionTemplate(ProcessingResourceTypeTemplate ...resources) {
		super(InternalAction.class);
		this.resources = Arrays.asList(resources);
	}

	@Override
	public boolean matches(InternalAction object) {
		if (object.getResourceDemand().size() != resources.size()) {
			return false;
		}
		for (ProcessingResourceTypeTemplate type : resources) {
			boolean found = false;
			for (ResourceDemand cur : object.getResourceDemand()) {
				if ((cur.getCharacterization() == ModelVariableCharacterizationType.EMPIRICAL)
						&& type.matches((ProcessingResourceType) cur.getResourceType())) {
					found = true;
					break;
				}
			}
			if (!found) {
				return false;
			}
		}
		return true;
	}

	@Override
	protected InternalAction create(ComponentInternalBehavior parent) {
		InternalAction action = ServicebehaviorFactory.eINSTANCE.createInternalAction();
		for (ProcessingResourceTypeTemplate res : resources) {
			ResourceDemand demand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
			demand.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
			demand.setResourceType(res.get(getContext().getResourceTypeRepository()));
			action.getResourceDemand().add(demand);
		}
		return action;
	}

}
